@@ -1,39 +1,50 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.transforms.i18n
     ~~~~~~~~~~~~~~~~~~~~~~
 
     Docutils transforms used by Sphinx when reading documents.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
+from textwrap import indent
+from typing import Any, TypeVar
 
 from docutils import nodes
 from docutils.io import StringInput
 from docutils.utils import relative_path
-from docutils.transforms import Transform
 
 from sphinx import addnodes
-from sphinx.util import split_index_msg
+from sphinx.domains.std import make_glossary_term, split_term_classifiers
+from sphinx.locale import __, init as init_locale
+from sphinx.transforms import SphinxTransform
+from sphinx.util import split_index_msg, logging
 from sphinx.util.i18n import find_catalog
 from sphinx.util.nodes import (
-    LITERAL_TYPE_NODES, IMAGE_TYPE_NODES,
+    LITERAL_TYPE_NODES, IMAGE_TYPE_NODES, NodeMatcher,
     extract_messages, is_pending_meta, traverse_translatable_index,
 )
-from sphinx.util.pycompat import indent
-from sphinx.locale import init as init_locale
-from sphinx.domains.std import make_glossary_term, split_term_classifiers
+
+if False:
+    # For type annotation
+    from typing import Dict, List, Tuple, Type  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config  # NOQA
+
+logger = logging.getLogger(__name__)
+
+N = TypeVar('N', bound=nodes.Node)
 
 
 def publish_msgstr(app, source, source_path, source_line, config, settings):
+    # type: (Sphinx, str, str, int, Config, Any) -> nodes.Element
     """Publish msgstr (single line) into docutils document
 
     :param sphinx.application.Sphinx app: sphinx application
-    :param unicode source: source text
-    :param unicode source_path: source path for warning indication
+    :param str source: source text
+    :param str source_path: source path for warning indication
     :param source_line: source line for warning indication
     :param sphinx.config.Config config: sphinx config
     :param docutils.frontend.Values settings: docutils settings
@@ -41,55 +52,54 @@ def publish_msgstr(app, source, source_path, source_line, config, settings):
     :rtype: docutils.nodes.document
     """
     from sphinx.io import SphinxI18nReader
-    reader = SphinxI18nReader(
-        app=app,
-        parsers=config.source_parsers,
-        parser_name='restructuredtext',  # default parser
-    )
-    reader.set_lineno_for_reporter(source_line)
+    reader = SphinxI18nReader(app)
+    parser = app.registry.create_source_parser(app, 'restructuredtext')
     doc = reader.read(
-        source=StringInput(source=source, source_path=source_path),
-        parser=reader.parser,
+        source=StringInput(source=source,
+                           source_path="%s:%s:<translated>" % (source_path, source_line)),
+        parser=parser,
         settings=settings,
     )
     try:
-        doc = doc[0]
+        doc = doc[0]  # type: ignore
     except IndexError:  # empty node
         pass
     return doc
 
 
-class PreserveTranslatableMessages(Transform):
+class PreserveTranslatableMessages(SphinxTransform):
     """
     Preserve original translatable messages befor translation
     """
     default_priority = 10  # this MUST be invoked before Locale transform
 
-    def apply(self):
+    def apply(self, **kwargs):
+        # type: (Any) -> None
         for node in self.document.traverse(addnodes.translatable):
             node.preserve_original_messages()
 
 
-class Locale(Transform):
+class Locale(SphinxTransform):
     """
     Replace translatable nodes with their translated doctree.
     """
     default_priority = 20
 
-    def apply(self):
-        env = self.document.settings.env
+    def apply(self, **kwargs):
+        # type: (Any) -> None
         settings, source = self.document.settings, self.document['source']
+        msgstr = ''
+
         # XXX check if this is reliable
-        assert source.startswith(env.srcdir)
-        docname = path.splitext(relative_path(path.join(env.srcdir, 'dummy'),
+        assert source.startswith(self.env.srcdir)
+        docname = path.splitext(relative_path(path.join(self.env.srcdir, 'dummy'),
                                               source))[0]
-        textdomain = find_catalog(docname,
-                                  self.document.settings.gettext_compact)
+        textdomain = find_catalog(docname, self.config.gettext_compact)
 
         # fetch translations
-        dirs = [path.join(env.srcdir, directory)
-                for directory in env.config.locale_dirs]
-        catalog, has_catalog = init_locale(dirs, env.config.language, textdomain)
+        dirs = [path.join(self.env.srcdir, directory)
+                for directory in self.config.locale_dirs]
+        catalog, has_catalog = init_locale(dirs, self.config.language, textdomain)
         if not has_catalog:
             return
 
@@ -114,8 +124,8 @@ class Locale(Transform):
             if isinstance(node, LITERAL_TYPE_NODES):
                 msgstr = '::\n\n' + indent(msgstr, ' ' * 3)
 
-            patch = publish_msgstr(
-                env.app, msgstr, source, node.line, env.config, settings)
+            patch = publish_msgstr(self.app, msgstr, source,
+                                   node.line, self.config, settings)
             # XXX doctest and other block markup
             if not isinstance(patch, nodes.paragraph):
                 continue  # skip for now
@@ -175,26 +185,24 @@ class Locale(Transform):
                         self.document.note_implicit_target(section_node)
 
                     # replace target's refname to new target name
-                    def is_named_target(node):
-                        return isinstance(node, nodes.target) and  \
-                            node.get('refname') == old_name
-                    for old_target in self.document.traverse(is_named_target):
+                    matcher = NodeMatcher(nodes.target, refname=old_name)
+                    for old_target in self.document.traverse(matcher):  # type: nodes.target
                         old_target['refname'] = new_name
 
                     processed = True
 
             # glossary terms update refid
             if isinstance(node, nodes.term):
-                gloss_entries = env.temp_data.setdefault('gloss_entries', set())
+                gloss_entries = self.env.temp_data.setdefault('gloss_entries', set())
                 for _id in node['names']:
                     if _id in gloss_entries:
                         gloss_entries.remove(_id)
 
                     parts = split_term_classifiers(msgstr)
-                    patch = publish_msgstr(
-                        env.app, parts[0], source, node.line, env.config, settings)
-                    patch = make_glossary_term(
-                        env, patch, parts[1], source, node.line, _id)
+                    patch = publish_msgstr(self.app, parts[0], source,
+                                           node.line, self.config, settings)
+                    patch = make_glossary_term(self.env, patch, parts[1],
+                                               source, node.line, _id)
                     node['ids'] = patch['ids']
                     node['names'] = patch['names']
                     processed = True
@@ -204,12 +212,12 @@ class Locale(Transform):
                 for child in patch.children:
                     child.parent = node
                 node.children = patch.children
-                node['translated'] = True
+                node['translated'] = True  # to avoid double translation
 
         # phase2: translation
         for node, msg in extract_messages(self.document):
-            if node.get('translated', False):
-                continue
+            if node.get('translated', False):  # to avoid double translation
+                continue  # skip if the node is already translated by phase1
 
             msgstr = catalog.gettext(msg)
             # XXX add marker to untranslated parts
@@ -222,7 +230,7 @@ class Locale(Transform):
                 continue
 
             # update meta nodes
-            if is_pending_meta(node):
+            if isinstance(node, nodes.pending) and is_pending_meta(node):
                 node.details['nodes'][0]['content'] = msgstr
                 continue
 
@@ -239,110 +247,165 @@ class Locale(Transform):
             if isinstance(node, LITERAL_TYPE_NODES):
                 msgstr = '::\n\n' + indent(msgstr, ' ' * 3)
 
-            patch = publish_msgstr(
-                env.app, msgstr, source, node.line, env.config, settings)
-            # XXX doctest and other block markup
-            if not isinstance(
-                    patch,
-                    (nodes.paragraph,) + LITERAL_TYPE_NODES + IMAGE_TYPE_NODES):
-                continue  # skip for now
+            # Structural Subelements phase1
+            # There is a possibility that only the title node is created.
+            # see: http://docutils.sourceforge.net/docs/ref/doctree.html#structural-subelements
+            if isinstance(node, nodes.title):
+                # This generates: <section ...><title>msgstr</title></section>
+                msgstr = msgstr + '\n' + '-' * len(msgstr) * 2
+
+            patch = publish_msgstr(self.app, msgstr, source,
+                                   node.line, self.config, settings)
+
+            # Structural Subelements phase2
+            if isinstance(node, nodes.title):
+                # get <title> node that placed as a first child
+                patch = patch.next_node()
+
+            # ignore unexpected markups in translation message
+            unexpected = (
+                nodes.paragraph,    # expected form of translation
+                nodes.title         # generated by above "Subelements phase2"
+            )  # type: Tuple[Type[nodes.Element], ...]
+
+            # following types are expected if
+            # config.gettext_additional_targets is configured
+            unexpected += LITERAL_TYPE_NODES
+            unexpected += IMAGE_TYPE_NODES
+
+            if not isinstance(patch, unexpected):
+                continue  # skip
 
             # auto-numbered foot note reference should use original 'ids'.
-            def is_autonumber_footnote_ref(node):
-                return isinstance(node, nodes.footnote_reference) and \
-                    node.get('auto') == 1
-
             def list_replace_or_append(lst, old, new):
+                # type: (List[N], N, N) -> None
                 if old in lst:
                     lst[lst.index(old)] = new
                 else:
                     lst.append(new)
-            old_foot_refs = node.traverse(is_autonumber_footnote_ref)
-            new_foot_refs = patch.traverse(is_autonumber_footnote_ref)
+
+            is_autofootnote_ref = NodeMatcher(nodes.footnote_reference, auto=Any)
+            old_foot_refs = node.traverse(is_autofootnote_ref)  # type: List[nodes.footnote_reference]  # NOQA
+            new_foot_refs = patch.traverse(is_autofootnote_ref)  # type: List[nodes.footnote_reference]  # NOQA
             if len(old_foot_refs) != len(new_foot_refs):
-                env.warn_node('inconsistent footnote references in '
-                              'translated message', node)
-            old_foot_namerefs = {}
+                old_foot_ref_rawsources = [ref.rawsource for ref in old_foot_refs]
+                new_foot_ref_rawsources = [ref.rawsource for ref in new_foot_refs]
+                logger.warning(__('inconsistent footnote references in translated message.' +
+                                  ' original: {0}, translated: {1}')
+                               .format(old_foot_ref_rawsources, new_foot_ref_rawsources),
+                               location=node)
+            old_foot_namerefs = {}  # type: Dict[str, List[nodes.footnote_reference]]
             for r in old_foot_refs:
                 old_foot_namerefs.setdefault(r.get('refname'), []).append(r)
-            for new in new_foot_refs:
-                refname = new.get('refname')
+            for newf in new_foot_refs:
+                refname = newf.get('refname')
                 refs = old_foot_namerefs.get(refname, [])
                 if not refs:
                     continue
 
-                old = refs.pop(0)
-                new['ids'] = old['ids']
-                for id in new['ids']:
-                    self.document.ids[id] = new
-                list_replace_or_append(
-                    self.document.autofootnote_refs, old, new)
+                oldf = refs.pop(0)
+                newf['ids'] = oldf['ids']
+                for id in newf['ids']:
+                    self.document.ids[id] = newf
+
+                if newf['auto'] == 1:
+                    # autofootnote_refs
+                    list_replace_or_append(self.document.autofootnote_refs, oldf, newf)
+                else:
+                    # symbol_footnote_refs
+                    list_replace_or_append(self.document.symbol_footnote_refs, oldf, newf)
+
                 if refname:
-                    list_replace_or_append(
-                        self.document.footnote_refs.setdefault(refname, []),
-                        old, new)
-                    list_replace_or_append(
-                        self.document.refnames.setdefault(refname, []),
-                        old, new)
+                    footnote_refs = self.document.footnote_refs.setdefault(refname, [])
+                    list_replace_or_append(footnote_refs, oldf, newf)
+
+                    refnames = self.document.refnames.setdefault(refname, [])
+                    list_replace_or_append(refnames, oldf, newf)
 
             # reference should use new (translated) 'refname'.
             # * reference target ".. _Python: ..." is not translatable.
             # * use translated refname for section refname.
             # * inline reference "`Python <...>`_" has no 'refname'.
-            def is_refnamed_ref(node):
-                return isinstance(node, nodes.reference) and  \
-                    'refname' in node
-            old_refs = node.traverse(is_refnamed_ref)
-            new_refs = patch.traverse(is_refnamed_ref)
+            is_refnamed_ref = NodeMatcher(nodes.reference, refname=Any)
+            old_refs = node.traverse(is_refnamed_ref)  # type: List[nodes.reference]
+            new_refs = patch.traverse(is_refnamed_ref)  # type: List[nodes.reference]
             if len(old_refs) != len(new_refs):
-                env.warn_node('inconsistent references in '
-                              'translated message', node)
+                old_ref_rawsources = [ref.rawsource for ref in old_refs]
+                new_ref_rawsources = [ref.rawsource for ref in new_refs]
+                logger.warning(__('inconsistent references in translated message.' +
+                                  ' original: {0}, translated: {1}')
+                               .format(old_ref_rawsources, new_ref_rawsources),
+                               location=node)
             old_ref_names = [r['refname'] for r in old_refs]
             new_ref_names = [r['refname'] for r in new_refs]
             orphans = list(set(old_ref_names) - set(new_ref_names))
-            for new in new_refs:
-                if not self.document.has_name(new['refname']):
+            for newr in new_refs:
+                if not self.document.has_name(newr['refname']):
                     # Maybe refname is translated but target is not translated.
                     # Note: multiple translated refnames break link ordering.
                     if orphans:
-                        new['refname'] = orphans.pop(0)
+                        newr['refname'] = orphans.pop(0)
                     else:
                         # orphan refnames is already empty!
                         # reference number is same in new_refs and old_refs.
                         pass
 
-                self.document.note_refname(new)
+                self.document.note_refname(newr)
 
-            # refnamed footnote and citation should use original 'ids'.
-            def is_refnamed_footnote_ref(node):
-                footnote_ref_classes = (nodes.footnote_reference,
-                                        nodes.citation_reference)
-                return isinstance(node, footnote_ref_classes) and \
-                    'refname' in node
-            old_refs = node.traverse(is_refnamed_footnote_ref)
-            new_refs = patch.traverse(is_refnamed_footnote_ref)
+            # refnamed footnote should use original 'ids'.
+            is_refnamed_footnote_ref = NodeMatcher(nodes.footnote_reference, refname=Any)
+            old_foot_refs = node.traverse(is_refnamed_footnote_ref)
+            new_foot_refs = patch.traverse(is_refnamed_footnote_ref)
+            refname_ids_map = {}  # type: Dict[str, List[str]]
+            if len(old_foot_refs) != len(new_foot_refs):
+                old_foot_ref_rawsources = [ref.rawsource for ref in old_foot_refs]
+                new_foot_ref_rawsources = [ref.rawsource for ref in new_foot_refs]
+                logger.warning(__('inconsistent footnote references in translated message.' +
+                                  ' original: {0}, translated: {1}')
+                               .format(old_foot_ref_rawsources, new_foot_ref_rawsources),
+                               location=node)
+            for oldf in old_foot_refs:
+                refname_ids_map.setdefault(oldf["refname"], []).append(oldf["ids"])
+            for newf in new_foot_refs:
+                refname = newf["refname"]
+                if refname_ids_map.get(refname):
+                    newf["ids"] = refname_ids_map[refname].pop(0)
+
+            # citation should use original 'ids'.
+            is_citation_ref = NodeMatcher(nodes.citation_reference, refname=Any)
+            old_cite_refs = node.traverse(is_citation_ref)  # type: List[nodes.citation_reference]  # NOQA
+            new_cite_refs = patch.traverse(is_citation_ref)  # type: List[nodes.citation_reference]  # NOQA
             refname_ids_map = {}
-            if len(old_refs) != len(new_refs):
-                env.warn_node('inconsistent references in '
-                              'translated message', node)
-            for old in old_refs:
-                refname_ids_map[old["refname"]] = old["ids"]
-            for new in new_refs:
-                refname = new["refname"]
-                if refname in refname_ids_map:
-                    new["ids"] = refname_ids_map[refname]
+            if len(old_cite_refs) != len(new_cite_refs):
+                old_cite_ref_rawsources = [ref.rawsource for ref in old_cite_refs]
+                new_cite_ref_rawsources = [ref.rawsource for ref in new_cite_refs]
+                logger.warning(__('inconsistent citation references in translated message.' +
+                                  ' original: {0}, translated: {1}')
+                               .format(old_cite_ref_rawsources, new_cite_ref_rawsources),
+                               location=node)
+            for oldc in old_cite_refs:
+                refname_ids_map.setdefault(oldc["refname"], []).append(oldc["ids"])
+            for newc in new_cite_refs:
+                refname = newc["refname"]
+                if refname_ids_map.get(refname):
+                    newc["ids"] = refname_ids_map[refname].pop()
 
             # Original pending_xref['reftarget'] contain not-translated
             # target name, new pending_xref must use original one.
             # This code restricts to change ref-targets in the translation.
-            old_refs = node.traverse(addnodes.pending_xref)
-            new_refs = patch.traverse(addnodes.pending_xref)
+            old_xrefs = node.traverse(addnodes.pending_xref)
+            new_xrefs = patch.traverse(addnodes.pending_xref)
             xref_reftarget_map = {}
-            if len(old_refs) != len(new_refs):
-                env.warn_node('inconsistent term references in '
-                              'translated message', node)
+            if len(old_xrefs) != len(new_xrefs):
+                old_xref_rawsources = [xref.rawsource for xref in old_xrefs]
+                new_xref_rawsources = [xref.rawsource for xref in new_xrefs]
+                logger.warning(__('inconsistent term references in translated message.' +
+                                  ' original: {0}, translated: {1}')
+                               .format(old_xref_rawsources, new_xref_rawsources),
+                               location=node)
 
             def get_ref_key(node):
+                # type: (addnodes.pending_xref) -> Tuple[str, str, str]
                 case = node["refdomain"], node["reftype"]
                 if case == ('std', 'term'):
                     return None
@@ -352,11 +415,11 @@ class Locale(Transform):
                         node["reftype"],
                         node['reftarget'],)
 
-            for old in old_refs:
+            for old in old_xrefs:
                 key = get_ref_key(old)
                 if key:
                     xref_reftarget_map[key] = old.attributes
-            for new in new_refs:
+            for new in new_xrefs:
                 key = get_ref_key(new)
                 # Copy attributes to keep original node behavior. Especially
                 # copying 'reftarget', 'py:module', 'py:class' are needed.
@@ -379,12 +442,12 @@ class Locale(Transform):
             if isinstance(node, IMAGE_TYPE_NODES):
                 node.update_all_atts(patch)
 
-            node['translated'] = True
+            node['translated'] = True  # to avoid double translation
 
-        if 'index' in env.config.gettext_additional_targets:
+        if 'index' in self.config.gettext_additional_targets:
             # Extract and translate messages for index entries.
             for node, entries in traverse_translatable_index(self.document):
-                new_entries = []
+                new_entries = []   # type: List[Tuple[str, str, str, str, str]]
                 for type, msg, tid, main, key_ in entries:
                     msg_parts = split_index_msg(type, msg)
                     msgstr_parts = []
@@ -399,20 +462,24 @@ class Locale(Transform):
                 node['raw_entries'] = entries
                 node['entries'] = new_entries
 
+        # remove translated attribute that is used for avoiding double translation.
+        for translated in self.document.traverse(NodeMatcher(translated=Any)):  # type: nodes.Element  # NOQA
+            translated.delattr('translated')
 
-class RemoveTranslatableInline(Transform):
+
+class RemoveTranslatableInline(SphinxTransform):
     """
     Remove inline nodes used for translation as placeholders.
     """
     default_priority = 999
 
-    def apply(self):
+    def apply(self, **kwargs):
+        # type: (Any) -> None
         from sphinx.builders.gettext import MessageCatalogBuilder
-        env = self.document.settings.env
-        builder = env.app.builder
-        if isinstance(builder, MessageCatalogBuilder):
+        if isinstance(self.app.builder, MessageCatalogBuilder):
             return
-        for inline in self.document.traverse(nodes.inline):
-            if 'translatable' in inline:
-                inline.parent.remove(inline)
-                inline.parent += inline.children
+
+        matcher = NodeMatcher(nodes.inline, translatable=Any)
+        for inline in self.document.traverse(matcher):  # type: nodes.inline
+            inline.parent.remove(inline)
+            inline.parent += inline.children
