@@ -1,18 +1,16 @@
-# -*- coding: utf-8 -*-
 """
     test_application
     ~~~~~~~~~~~~~~~~
 
     Test the Sphinx class.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import pytest
 from docutils import nodes
 
-from sphinx.application import ExtensionError
-from sphinx.domains import Domain
+from sphinx.errors import ExtensionError
 from sphinx.testing.util import strip_escseq
 from sphinx.util import logging
 
@@ -43,13 +41,14 @@ def test_events(app, status, warning):
 
 
 def test_emit_with_nonascii_name_node(app, status, warning):
-    node = nodes.section(names=[u'\u65e5\u672c\u8a9e'])
+    node = nodes.section(names=['\u65e5\u672c\u8a9e'])
     app.emit('my_event', node)
 
 
 def test_extensions(app, status, warning):
     app.setup_extension('shutil')
-    assert strip_escseq(warning.getvalue()).startswith("WARNING: extension 'shutil'")
+    warning = strip_escseq(warning.getvalue())
+    assert "extension 'shutil' has no setup() function" in warning
 
 
 def test_extension_in_blacklist(app, status, warning):
@@ -58,34 +57,25 @@ def test_extension_in_blacklist(app, status, warning):
     assert msg.startswith("WARNING: the extension 'sphinxjp.themecore' was")
 
 
-def test_domain_override(app, status, warning):
-    class A(Domain):
-        name = 'foo'
-
-    class B(A):
-        name = 'foo'
-
-    class C(Domain):
-        name = 'foo'
-
-    # No domain know named foo.
-    with pytest.raises(ExtensionError) as excinfo:
-        app.override_domain(A)
-    assert 'domain foo not yet registered' in str(excinfo.value)
-
-    assert app.add_domain(A) is None
-    assert app.override_domain(B) is None
-    with pytest.raises(ExtensionError) as excinfo:
-        app.override_domain(C)
-    assert 'new domain not a subclass of registered foo domain' in str(excinfo.value)
-
-
 @pytest.mark.sphinx(testroot='add_source_parser')
+@pytest.mark.filterwarnings('ignore:The config variable "source_parsers"')
+@pytest.mark.filterwarnings('ignore:app.add_source_parser\\(\\) does not support suffix')
 def test_add_source_parser(app, status, warning):
-    assert set(app.config.source_suffix) == set(['.rst', '.md', '.test'])
-    assert set(app.registry.get_source_parsers().keys()) == set(['*', '.md', '.test'])
+    assert set(app.config.source_suffix) == {'.rst', '.md', '.test'}
+
+    # .rst; only in :confval:`source_suffix`
+    assert '.rst' not in app.registry.get_source_parsers()
+    assert app.registry.source_suffix['.rst'] is None
+
+    # .md; configured by :confval:`source_suffix` and :confval:`source_parsers`
+    assert '.md' in app.registry.get_source_parsers()
+    assert app.registry.source_suffix['.md'] == '.md'
     assert app.registry.get_source_parsers()['.md'].__name__ == 'DummyMarkdownParser'
-    assert app.registry.get_source_parsers()['.test'].__name__ == 'TestSourceParser'
+
+    # .test; configured by API
+    assert app.registry.source_suffix['.test'] == 'test'
+    assert 'test' in app.registry.get_source_parsers()
+    assert app.registry.get_source_parsers()['test'].__name__ == 'TestSourceParser'
 
 
 @pytest.mark.sphinx(testroot='extensions')
