@@ -1,22 +1,23 @@
-# -*- coding: utf-8 -*-
 """
     test_domain_std
     ~~~~~~~~~~~~~~~
 
     Tests the std domain
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from unittest import mock
+
 from docutils import nodes
-import mock
 
 from sphinx.domains.std import StandardDomain
 
 
 def test_process_doc_handle_figure_caption():
     env = mock.Mock(domaindata={})
+    env.app.registry.enumerable_nodes = {}
     figure_node = nodes.figure(
         '',
         nodes.caption('caption text', 'caption text'),
@@ -40,6 +41,7 @@ def test_process_doc_handle_figure_caption():
 
 def test_process_doc_handle_table_title():
     env = mock.Mock(domaindata={})
+    env.app.registry.enumerable_nodes = {}
     table_node = nodes.table(
         '',
         nodes.title('title text', 'title text'),
@@ -63,6 +65,7 @@ def test_process_doc_handle_table_title():
 
 def test_get_full_qualified_name():
     env = mock.Mock(domaindata={})
+    env.app.registry.enumerable_nodes = {}
     domain = StandardDomain(env)
 
     # normal references
