@@ -1,18 +1,18 @@
-# -*- coding: utf-8 -*-
 """
     test_highlighting
     ~~~~~~~~~~~~~~~~~
 
     Test the Pygments highlighting bridge.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from unittest import mock
+
+from pygments.formatters.html import HtmlFormatter
 from pygments.lexer import RegexLexer
 from pygments.token import Text, Name
-from pygments.filters import ErrorToken
-from pygments.formatters.html import HtmlFormatter
 
 from sphinx.highlighting import PygmentsBridge
 
@@ -76,17 +76,8 @@ def test_set_formatter():
         PygmentsBridge.html_formatter = HtmlFormatter
 
 
-def test_trim_doctest_flags():
-    PygmentsBridge.html_formatter = MyFormatter
-    try:
-        bridge = PygmentsBridge('html', trim_doctest_flags=True)
-        ret = bridge.highlight_block('>>> 1+2 # doctest: SKIP\n3\n', 'pycon')
-        assert ret == '>>> 1+2 \n3\n'
-    finally:
-        PygmentsBridge.html_formatter = HtmlFormatter
-
-
-def test_default_highlight():
+@mock.patch('sphinx.highlighting.logger')
+def test_default_highlight(logger):
     bridge = PygmentsBridge('html')
 
     # default: highlights as python3
@@ -104,8 +95,8 @@ def test_default_highlight():
                    '<span class="s2">&quot;Hello sphinx world&quot;</span>\n</pre></div>\n')
 
     # python3: raises error if highlighting failed
-    try:
-        ret = bridge.highlight_block('reST ``like`` text', 'python3')
-        assert False, "highlight_block() does not raise any exceptions"
-    except ErrorToken:
-        pass  # raise parsing error
+    ret = bridge.highlight_block('reST ``like`` text', 'python3')
+    logger.warning.assert_called_with('Could not lex literal_block as "%s". '
+                                      'Highlighting skipped.', 'python3',
+                                      type='misc', subtype='highlighting_failure',
+                                      location=None)
