@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     test_intl
     ~~~~~~~~~
@@ -6,22 +5,20 @@
     Test message patching for internationalization purposes.  Runs the text
     builder in the test root.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from __future__ import print_function
 
 import os
 import re
-import pickle
+
+import pytest
+from babel.messages import pofile, mofile
 from docutils import nodes
 
-from babel.messages import pofile, mofile
-from six import string_types
-import pytest
-
-from util import (
-    path, assert_re_search, assert_not_re_search, assert_startswith, assert_node, etree_parse
+from sphinx.testing.util import (
+    path, etree_parse, strip_escseq,
+    assert_re_search, assert_not_re_search, assert_startswith, assert_node
 )
 
 
@@ -60,7 +57,9 @@ def build_mo():
                 if not mo.parent.exists():
                     mo.parent.makedirs()
 
-                write_mo(mo, read_po(po))
+                if not mo.exists() or mo.stat().st_mtime < po.stat().st_mtime:
+                    # compile .mo file only if needed
+                    write_mo(mo, read_po(po))
     return builder
 
 
@@ -77,20 +76,7 @@ def _info(app):
 
 
 def elem_gettexts(elem):
-    def itertext(self):
-        # this function copied from Python-2.7 'ElementTree.itertext'.
-        # for compatibility to Python-2.6
-        tag = self.tag
-        if not isinstance(tag, string_types) and tag is not None:
-            return
-        if self.text:
-            yield self.text
-        for e in self:
-            for s in itertext(e):
-                yield s
-            if e.tail:
-                yield e.tail
-    return [_f for _f in [s.strip() for s in itertext(elem)] if _f]
+    return [_f for _f in [s.strip() for s in elem.itertext()] if _f]
 
 
 def elem_getref(elem):
@@ -119,8 +105,8 @@ def assert_count(expected_expr, result, count):
 @pytest.mark.test_params(shared_result='test_intl_basic')
 def test_text_toctree(app):
     app.build()
-    result = (app.outdir / 'contents.txt').text(encoding='utf-8')
-    assert_startswith(result, u"CONTENTS\n********\n\nTABLE OF CONTENTS\n")
+    result = (app.outdir / 'index.txt').text()
+    assert_startswith(result, "CONTENTS\n********\n\nTABLE OF CONTENTS\n")
 
 
 @sphinx_intl
@@ -130,8 +116,8 @@ def test_text_emit_warnings(app, warning):
     app.build()
     # test warnings in translation
     warnings = getwarning(warning)
-    warning_expr = u'.*/warnings.txt:4: ' \
-                   u'WARNING: Inline literal start-string without end-string.\n'
+    warning_expr = ('.*/warnings.txt:4:<translated>:1: '
+                   'WARNING: Inline literal start-string without end-string.\n')
     assert_re_search(warning_expr, warnings)
 
 
@@ -141,23 +127,24 @@ def test_text_emit_warnings(app, warning):
 def test_text_warning_node(app):
     app.build()
     # test warnings in translation
-    result = (app.outdir / 'warnings.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH REST WARNINGS"
-              u"\n***********************\n"
-              u"\nLINE OF >>``<<BROKEN LITERAL MARKUP.\n")
+    result = (app.outdir / 'warnings.txt').text()
+    expect = ("3. I18N WITH REST WARNINGS"
+              "\n**************************\n"
+              "\nLINE OF >>``<<BROKEN LITERAL MARKUP.\n")
     assert result == expect
 
 
 @sphinx_intl
 @pytest.mark.sphinx('text')
 @pytest.mark.test_params(shared_result='test_intl_basic')
+@pytest.mark.xfail(os.name != 'posix', reason="Not working on windows")
 def test_text_title_underline(app):
     app.build()
     # --- simple translation; check title underlines
-    result = (app.outdir / 'bom.txt').text(encoding='utf-8')
-    expect = (u"Datei mit UTF-8"
-              u"\n***************\n"  # underline matches new translation
-              u"\nThis file has umlauts: äöü.\n")
+    result = (app.outdir / 'bom.txt').text()
+    expect = ("2. Datei mit UTF-8"
+              "\n******************\n"  # underline matches new translation
+              "\nThis file has umlauts: äöü.\n")
     assert result == expect
 
 
@@ -167,8 +154,8 @@ def test_text_title_underline(app):
 def test_text_subdirs(app):
     app.build()
     # --- check translation in subdirs
-    result = (app.outdir / 'subdir' / 'contents.txt').text(encoding='utf-8')
-    assert_startswith(result, u"subdir contents\n***************\n")
+    result = (app.outdir / 'subdir' / 'index.txt').text()
+    assert_startswith(result, "1. subdir contents\n******************\n")
 
 
 @sphinx_intl
@@ -177,26 +164,49 @@ def test_text_subdirs(app):
 def test_text_inconsistency_warnings(app, warning):
     app.build()
     # --- check warnings for inconsistency in number of references
-    result = (app.outdir / 'refs_inconsistency.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH REFS INCONSISTENCY"
-              u"\n****************************\n"
-              u"\n* FOR FOOTNOTE [ref2].\n"
-              u"\n* reference FOR reference.\n"
-              u"\n* ORPHAN REFERENCE: I18N WITH REFS INCONSISTENCY.\n"
-              u"\n[1] THIS IS A AUTO NUMBERED FOOTNOTE.\n"
-              u"\n[ref2] THIS IS A NAMED FOOTNOTE.\n"
-              u"\n[100] THIS IS A NUMBERED FOOTNOTE.\n")
+    result = (app.outdir / 'refs_inconsistency.txt').text()
+    expect = ("8. I18N WITH REFS INCONSISTENCY"
+              "\n*******************************\n"
+              "\n* FOR CITATION [ref3].\n"
+              "\n* reference FOR reference.\n"
+              "\n* ORPHAN REFERENCE: I18N WITH REFS INCONSISTENCY.\n"
+              "\n[1] THIS IS A AUTO NUMBERED FOOTNOTE.\n"
+              "\n[ref2] THIS IS A CITATION.\n"
+              "\n[100] THIS IS A NUMBERED FOOTNOTE.\n")
     assert result == expect
 
     warnings = getwarning(warning)
-    warning_fmt = u'.*/refs_inconsistency.txt:\\d+: ' \
-                  u'WARNING: inconsistent %s in translated message\n'
+    warning_fmt = ('.*/refs_inconsistency.txt:\\d+: '
+                   'WARNING: inconsistent %(reftype)s in translated message.'
+                   ' original: %(original)s, translated: %(translated)s\n')
     expected_warning_expr = (
-        warning_fmt % 'footnote references' +
-        warning_fmt % 'references' +
-        warning_fmt % 'references')
+        warning_fmt % {
+            'reftype': 'footnote references',
+            'original': "\\['\\[#\\]_'\\]",
+            'translated': "\\[\\]"
+        } +
+        warning_fmt % {
+            'reftype': 'footnote references',
+            'original': "\\['\\[100\\]_'\\]",
+            'translated': "\\[\\]"
+        } +
+        warning_fmt % {
+            'reftype': 'references',
+            'original': "\\['reference_'\\]",
+            'translated': "\\['reference_', 'reference_'\\]"
+        } +
+        warning_fmt % {
+            'reftype': 'references',
+            'original': "\\[\\]",
+            'translated': "\\['`I18N WITH REFS INCONSISTENCY`_'\\]"
+        })
     assert_re_search(expected_warning_expr, warnings)
 
+    expected_citation_warning_expr = (
+        '.*/refs_inconsistency.txt:\\d+: WARNING: Citation \\[ref2\\] is not referenced.\n' +
+        '.*/refs_inconsistency.txt:\\d+: WARNING: citation not found: ref3')
+    assert_re_search(expected_citation_warning_expr, warnings)
+
 
 @sphinx_intl
 @pytest.mark.sphinx('text')
@@ -204,19 +214,19 @@ def test_text_inconsistency_warnings(app, warning):
 def test_text_literalblock_warnings(app, warning):
     app.build()
     # --- check warning for literal block
-    result = (app.outdir / 'literalblock.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH LITERAL BLOCK"
-              u"\n***********************\n"
-              u"\nCORRECT LITERAL BLOCK:\n"
-              u"\n   this is"
-              u"\n   literal block\n"
-              u"\nMISSING LITERAL BLOCK:\n"
-              u"\n<SYSTEM MESSAGE:")
+    result = (app.outdir / 'literalblock.txt').text()
+    expect = ("9. I18N WITH LITERAL BLOCK"
+              "\n**************************\n"
+              "\nCORRECT LITERAL BLOCK:\n"
+              "\n   this is"
+              "\n   literal block\n"
+              "\nMISSING LITERAL BLOCK:\n"
+              "\n<SYSTEM MESSAGE:")
     assert_startswith(result, expect)
 
     warnings = getwarning(warning)
-    expected_warning_expr = u'.*/literalblock.txt:\\d+: ' \
-                            u'WARNING: Literal block expected; none found.'
+    expected_warning_expr = ('.*/literalblock.txt:\\d+: '
+                             'WARNING: Literal block expected; none found.')
     assert_re_search(expected_warning_expr, warnings)
 
 
@@ -226,18 +236,17 @@ def test_text_literalblock_warnings(app, warning):
 def test_text_definition_terms(app):
     app.build()
     # --- definition terms: regression test for #975, #2198, #2205
-    result = (app.outdir / 'definition_terms.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH DEFINITION TERMS"
-              u"\n**************************\n"
-              u"\nSOME TERM"
-              u"\n   THE CORRESPONDING DEFINITION\n"
-              u"\nSOME *TERM* WITH LINK"
-              u"\n   THE CORRESPONDING DEFINITION #2\n"
-              u"\nSOME **TERM** WITH : CLASSIFIER1 : CLASSIFIER2"
-              u"\n   THE CORRESPONDING DEFINITION\n"
-              u"\nSOME TERM WITH : CLASSIFIER[]"
-              u"\n   THE CORRESPONDING DEFINITION\n"
-              )
+    result = (app.outdir / 'definition_terms.txt').text()
+    expect = ("13. I18N WITH DEFINITION TERMS"
+              "\n******************************\n"
+              "\nSOME TERM"
+              "\n   THE CORRESPONDING DEFINITION\n"
+              "\nSOME *TERM* WITH LINK"
+              "\n   THE CORRESPONDING DEFINITION #2\n"
+              "\nSOME **TERM** WITH : CLASSIFIER1 : CLASSIFIER2"
+              "\n   THE CORRESPONDING DEFINITION\n"
+              "\nSOME TERM WITH : CLASSIFIER[]"
+              "\n   THE CORRESPONDING DEFINITION\n")
     assert result == expect
 
 
@@ -247,14 +256,14 @@ def test_text_definition_terms(app):
 def test_text_glossary_term(app, warning):
     app.build()
     # --- glossary terms: regression test for #1090
-    result = (app.outdir / 'glossary_terms.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH GLOSSARY TERMS"
-              u"\n************************\n"
-              u"\nSOME NEW TERM"
-              u"\n   THE CORRESPONDING GLOSSARY\n"
-              u"\nSOME OTHER NEW TERM"
-              u"\n   THE CORRESPONDING GLOSSARY #2\n"
-              u"\nLINK TO *SOME NEW TERM*.\n")
+    result = (app.outdir / 'glossary_terms.txt').text()
+    expect = ("18. I18N WITH GLOSSARY TERMS"
+              "\n****************************\n"
+              "\nSOME NEW TERM"
+              "\n   THE CORRESPONDING GLOSSARY\n"
+              "\nSOME OTHER NEW TERM"
+              "\n   THE CORRESPONDING GLOSSARY #2\n"
+              "\nLINK TO *SOME NEW TERM*.\n")
     assert result == expect
     warnings = getwarning(warning)
     assert 'term not in glossary' not in warnings
@@ -266,32 +275,58 @@ def test_text_glossary_term(app, warning):
 def test_text_glossary_term_inconsistencies(app, warning):
     app.build()
     # --- glossary term inconsistencies: regression test for #1090
-    result = (app.outdir / 'glossary_terms_inconsistency.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH GLOSSARY TERMS INCONSISTENCY"
-              u"\n**************************************\n"
-              u"\n1. LINK TO *SOME NEW TERM*.\n")
+    result = (app.outdir / 'glossary_terms_inconsistency.txt').text()
+    expect = ("19. I18N WITH GLOSSARY TERMS INCONSISTENCY"
+              "\n******************************************\n"
+              "\n1. LINK TO *SOME NEW TERM*.\n")
     assert result == expect
 
     warnings = getwarning(warning)
     expected_warning_expr = (
-        u'.*/glossary_terms_inconsistency.txt:\\d+: '
-        u'WARNING: inconsistent term references in translated message\n')
+        '.*/glossary_terms_inconsistency.txt:\\d+: '
+        'WARNING: inconsistent term references in translated message.'
+        " original: \\[':term:`Some term`', ':term:`Some other term`'\\],"
+        " translated: \\[':term:`SOME NEW TERM`'\\]\n")
     assert_re_search(expected_warning_expr, warnings)
 
 
+@sphinx_intl
+@pytest.mark.sphinx('gettext')
+@pytest.mark.test_params(shared_result='test_intl_gettext')
+def test_gettext_section(app):
+    app.build()
+    # --- section
+    expect = read_po(app.srcdir / 'section.po')
+    actual = read_po(app.outdir / 'section.pot')
+    for expect_msg in [m for m in expect if m.id]:
+        assert expect_msg.id in [m.id for m in actual if m.id]
+
+
+@sphinx_intl
+@pytest.mark.sphinx('text')
+@pytest.mark.test_params(shared_result='test_intl_basic')
+def test_text_section(app):
+    app.build()
+    # --- section
+    result = (app.outdir / 'section.txt').text()
+    expect = read_po(app.srcdir / 'section.po')
+    for expect_msg in [m for m in expect if m.id]:
+        assert expect_msg.string in result
+
+
 @sphinx_intl
 @pytest.mark.sphinx('text')
 @pytest.mark.test_params(shared_result='test_intl_basic')
 def test_text_seealso(app):
     app.build()
     # --- seealso
-    result = (app.outdir / 'seealso.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH SEEALSO"
-              u"\n*****************\n"
-              u"\nSee also: SHORT TEXT 1\n"
-              u"\nSee also: LONG TEXT 1\n"
-              u"\nSee also: SHORT TEXT 2\n"
-              u"\n  LONG TEXT 2\n")
+    result = (app.outdir / 'seealso.txt').text()
+    expect = ("12. I18N WITH SEEALSO"
+              "\n*********************\n"
+              "\nSee also: SHORT TEXT 1\n"
+              "\nSee also: LONG TEXT 1\n"
+              "\nSee also: SHORT TEXT 2\n"
+              "\n  LONG TEXT 2\n")
     assert result == expect
 
 
@@ -301,40 +336,39 @@ def test_text_seealso(app):
 def test_text_figure_captions(app):
     app.build()
     # --- figure captions: regression test for #940
-    result = (app.outdir / 'figure.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH FIGURE CAPTION"
-              u"\n************************\n"
-              u"\n   [image]MY CAPTION OF THE FIGURE\n"
-              u"\n   MY DESCRIPTION PARAGRAPH1 OF THE FIGURE.\n"
-              u"\n   MY DESCRIPTION PARAGRAPH2 OF THE FIGURE.\n"
-              u"\n"
-              u"\nFIGURE IN THE BLOCK"
-              u"\n===================\n"
-              u"\nBLOCK\n"
-              u"\n      [image]MY CAPTION OF THE FIGURE\n"
-              u"\n      MY DESCRIPTION PARAGRAPH1 OF THE FIGURE.\n"
-              u"\n      MY DESCRIPTION PARAGRAPH2 OF THE FIGURE.\n"
-              u"\n"
-              u"\n"
-              u"IMAGE URL AND ALT\n"
-              u"=================\n"
-              u"\n"
-              u"[image: i18n][image]\n"
-              u"\n"
-              u"   [image: img][image]\n"
-              u"\n"
-              u"\n"
-              u"IMAGE ON SUBSTITUTION\n"
-              u"=====================\n"
-              u"\n"
-              u"\n"
-              u"IMAGE UNDER NOTE\n"
-              u"================\n"
-              u"\n"
-              u"Note: [image: i18n under note][image]\n"
-              u"\n"
-              u"     [image: img under note][image]\n"
-              )
+    result = (app.outdir / 'figure.txt').text()
+    expect = ("14. I18N WITH FIGURE CAPTION"
+              "\n****************************\n"
+              "\n   [image]MY CAPTION OF THE FIGURE\n"
+              "\n   MY DESCRIPTION PARAGRAPH1 OF THE FIGURE.\n"
+              "\n   MY DESCRIPTION PARAGRAPH2 OF THE FIGURE.\n"
+              "\n"
+              "\n14.1. FIGURE IN THE BLOCK"
+              "\n=========================\n"
+              "\nBLOCK\n"
+              "\n      [image]MY CAPTION OF THE FIGURE\n"
+              "\n      MY DESCRIPTION PARAGRAPH1 OF THE FIGURE.\n"
+              "\n      MY DESCRIPTION PARAGRAPH2 OF THE FIGURE.\n"
+              "\n"
+              "\n"
+              "14.2. IMAGE URL AND ALT\n"
+              "=======================\n"
+              "\n"
+              "[image: i18n][image]\n"
+              "\n"
+              "   [image: img][image]\n"
+              "\n"
+              "\n"
+              "14.3. IMAGE ON SUBSTITUTION\n"
+              "===========================\n"
+              "\n"
+              "\n"
+              "14.4. IMAGE UNDER NOTE\n"
+              "======================\n"
+              "\n"
+              "Note: [image: i18n under note][image]\n"
+              "\n"
+              "     [image: img under note][image]\n")
     assert result == expect
 
 
@@ -344,15 +378,15 @@ def test_text_figure_captions(app):
 def test_text_rubric(app):
     app.build()
     # --- rubric: regression test for pull request #190
-    result = (app.outdir / 'rubric.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH RUBRIC"
-              u"\n****************\n"
-              u"\n-[ RUBRIC TITLE ]-\n"
-              u"\n"
-              u"\nRUBRIC IN THE BLOCK"
-              u"\n===================\n"
-              u"\nBLOCK\n"
-              u"\n   -[ RUBRIC TITLE ]-\n")
+    result = (app.outdir / 'rubric.txt').text()
+    expect = ("I18N WITH RUBRIC"
+              "\n****************\n"
+              "\n-[ RUBRIC TITLE ]-\n"
+              "\n"
+              "\nRUBRIC IN THE BLOCK"
+              "\n===================\n"
+              "\nBLOCK\n"
+              "\n   -[ RUBRIC TITLE ]-\n")
     assert result == expect
 
 
@@ -362,26 +396,26 @@ def test_text_rubric(app):
 def test_text_docfields(app):
     app.build()
     # --- docfields
-    result = (app.outdir / 'docfields.txt').text(encoding='utf-8')
-    expect = (u"I18N WITH DOCFIELDS"
-              u"\n*******************\n"
-              u"\nclass Cls1\n"
-              u"\n   Parameters:"
-              u"\n      **param** -- DESCRIPTION OF PARAMETER param\n"
-              u"\nclass Cls2\n"
-              u"\n   Parameters:"
-              u"\n      * **foo** -- DESCRIPTION OF PARAMETER foo\n"
-              u"\n      * **bar** -- DESCRIPTION OF PARAMETER bar\n"
-              u"\nclass Cls3(values)\n"
-              u"\n   Raises:"
-              u"\n      **ValueError** -- IF THE VALUES ARE OUT OF RANGE\n"
-              u"\nclass Cls4(values)\n"
-              u"\n   Raises:"
-              u"\n      * **TypeError** -- IF THE VALUES ARE NOT VALID\n"
-              u"\n      * **ValueError** -- IF THE VALUES ARE OUT OF RANGE\n"
-              u"\nclass Cls5\n"
-              u"\n   Returns:"
-              u'\n      A NEW "Cls3" INSTANCE\n')
+    result = (app.outdir / 'docfields.txt').text()
+    expect = ("21. I18N WITH DOCFIELDS"
+              "\n***********************\n"
+              "\nclass Cls1\n"
+              "\n   Parameters:"
+              "\n      **param** -- DESCRIPTION OF PARAMETER param\n"
+              "\nclass Cls2\n"
+              "\n   Parameters:"
+              "\n      * **foo** -- DESCRIPTION OF PARAMETER foo\n"
+              "\n      * **bar** -- DESCRIPTION OF PARAMETER bar\n"
+              "\nclass Cls3(values)\n"
+              "\n   Raises:"
+              "\n      **ValueError** -- IF THE VALUES ARE OUT OF RANGE\n"
+              "\nclass Cls4(values)\n"
+              "\n   Raises:"
+              "\n      * **TypeError** -- IF THE VALUES ARE NOT VALID\n"
+              "\n      * **ValueError** -- IF THE VALUES ARE OUT OF RANGE\n"
+              "\nclass Cls5\n"
+              "\n   Returns:"
+              '\n      A NEW "Cls3" INSTANCE\n')
     assert result == expect
 
 
@@ -393,7 +427,7 @@ def test_text_admonitions(app):
     # --- admonitions
     # #1206: gettext did not translate admonition directive's title
     # seealso: http://docutils.sourceforge.net/docs/ref/rst/directives.html#admonitions
-    result = (app.outdir / 'admonitions.txt').text(encoding='utf-8')
+    result = (app.outdir / 'admonitions.txt').text()
     directives = (
         "attention", "caution", "danger", "error", "hint",
         "important", "note", "tip", "warning", "admonition")
@@ -401,6 +435,9 @@ def test_text_admonitions(app):
         assert d.upper() + " TITLE" in result
         assert d.upper() + " BODY" in result
 
+    # for #4938 `1. ` prefixed admonition title
+    assert "1. ADMONITION TITLE" in result
+
 
 @sphinx_intl
 @pytest.mark.sphinx('gettext')
@@ -408,8 +445,8 @@ def test_text_admonitions(app):
 def test_gettext_toctree(app):
     app.build()
     # --- toctree
-    expect = read_po(app.srcdir / 'contents.po')
-    actual = read_po(app.outdir / 'contents.pot')
+    expect = read_po(app.srcdir / 'index.po')
+    actual = read_po(app.outdir / 'index.pot')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.id in [m.id for m in actual if m.id]
 
@@ -426,6 +463,42 @@ def test_gettext_table(app):
         assert expect_msg.id in [m.id for m in actual if m.id]
 
 
+@sphinx_intl
+@pytest.mark.sphinx('text')
+@pytest.mark.test_params(shared_result='test_intl_basic')
+def test_text_table(app):
+    app.build()
+    # --- toctree
+    result = (app.outdir / 'table.txt').text()
+    expect = read_po(app.srcdir / 'table.po')
+    for expect_msg in [m for m in expect if m.id]:
+        assert expect_msg.string in result
+
+
+@sphinx_intl
+@pytest.mark.sphinx('gettext')
+@pytest.mark.test_params(shared_result='test_intl_gettext')
+def test_gettext_topic(app):
+    app.build()
+    # --- topic
+    expect = read_po(app.srcdir / 'topic.po')
+    actual = read_po(app.outdir / 'topic.pot')
+    for expect_msg in [m for m in expect if m.id]:
+        assert expect_msg.id in [m.id for m in actual if m.id]
+
+
+@sphinx_intl
+@pytest.mark.sphinx('text')
+@pytest.mark.test_params(shared_result='test_intl_basic')
+def test_text_topic(app):
+    app.build()
+    # --- topic
+    result = (app.outdir / 'topic.txt').text()
+    expect = read_po(app.srcdir / 'topic.po')
+    for expect_msg in [m for m in expect if m.id]:
+        assert expect_msg.string in result
+
+
 @sphinx_intl
 @pytest.mark.sphinx('gettext')
 @pytest.mark.test_params(shared_result='test_intl_gettext')
@@ -493,21 +566,22 @@ def test_gettext_buildr_ignores_only_directive(app):
 
 
 @sphinx_intl
+# use individual shared_result directory to avoid "incompatible doctree" error
+@pytest.mark.sphinx(testroot='builder-gettext-dont-rebuild-mo')
 def test_gettext_dont_rebuild_mo(make_app, app_params, build_mo):
     # --- don't rebuild by .mo mtime
     def get_number_of_update_targets(app_):
-        updated = app_.env.update(app_.config, app_.srcdir, app_.doctreedir, app_)
+        app_.env.find_files(app_.config, app_.builder)
+        _, updated, _ = app_.env.get_outdated_files(config_changed=False)
         return len(updated)
 
-    # setup new directory
     args, kwargs = app_params
-    kwargs['srcdir'] = 'test_gettext_dont_rebuild_mo'
 
     # phase1: build document with non-gettext builder and generate mo file in srcdir
     app0 = make_app('dummy', *args, **kwargs)
     build_mo(app0.srcdir)
     app0.build()
-    assert (app0.srcdir / 'bom.mo')
+    assert (app0.srcdir / 'xx' / 'LC_MESSAGES' / 'bom.mo').exists()
     # Since it is after the build, the number of documents to be updated is 0
     assert get_number_of_update_targets(app0) == 0
     # When rewriting the timestamp of mo file, the number of documents to be
@@ -538,11 +612,13 @@ def test_gettext_dont_rebuild_mo(make_app, app_params, build_mo):
 def test_html_meta(app):
     app.build()
     # --- test for meta
-    result = (app.outdir / 'contents.html').text(encoding='utf-8')
+    result = (app.outdir / 'index.html').text()
     expected_expr = '<meta content="TESTDATA FOR I18N" name="description" />'
     assert expected_expr in result
     expected_expr = '<meta content="I18N, SPHINX, MARKUP" name="keywords" />'
     assert expected_expr in result
+    expected_expr = '<p class="caption"><span class="caption-text">HIDDEN TOC</span></p>'
+    assert expected_expr in result
 
 
 @sphinx_intl
@@ -552,7 +628,7 @@ def test_html_footnotes(app):
     app.build()
     # --- test for #955 cant-build-html-with-footnotes-when-using
     # expect no error by build
-    (app.outdir / 'footnote.html').text(encoding='utf-8')
+    (app.outdir / 'footnote.html').text()
 
 
 @sphinx_intl
@@ -561,7 +637,7 @@ def test_html_footnotes(app):
 def test_html_undefined_refs(app):
     app.build()
     # --- links to undefined reference
-    result = (app.outdir / 'refs_inconsistency.html').text(encoding='utf-8')
+    result = (app.outdir / 'refs_inconsistency.html').text()
 
     expected_expr = ('<a class="reference external" '
                      'href="http://www.example.com">reference</a>')
@@ -583,7 +659,7 @@ def test_html_undefined_refs(app):
 def test_html_index_entries(app):
     app.build()
     # --- index entries: regression test for #976
-    result = (app.outdir / 'genindex.html').text(encoding='utf-8')
+    result = (app.outdir / 'genindex.html').text()
 
     def wrap(tag, keyword):
         start_tag = "<%s[^>]*>" % tag
@@ -621,7 +697,7 @@ def test_html_index_entries(app):
 def test_html_versionchanges(app):
     app.build()
     # --- versionchanges
-    result = (app.outdir / 'versionchange.html').text(encoding='utf-8')
+    result = (app.outdir / 'versionchange.html').text()
 
     def get_content(result, name):
         matched = re.search(r'<div class="%s">\n*(.*?)</div>' % name,
@@ -632,21 +708,21 @@ def test_html_versionchanges(app):
             return ''
 
     expect1 = (
-        u"""<p><span class="versionmodified">Deprecated since version 1.0: </span>"""
-        u"""THIS IS THE <em>FIRST</em> PARAGRAPH OF DEPRECATED.</p>\n"""
-        u"""<p>THIS IS THE <em>SECOND</em> PARAGRAPH OF DEPRECATED.</p>\n""")
+        """<p><span class="versionmodified deprecated">Deprecated since version 1.0: </span>"""
+        """THIS IS THE <em>FIRST</em> PARAGRAPH OF DEPRECATED.</p>\n"""
+        """<p>THIS IS THE <em>SECOND</em> PARAGRAPH OF DEPRECATED.</p>\n""")
     matched_content = get_content(result, "deprecated")
     assert expect1 == matched_content
 
     expect2 = (
-        u"""<p><span class="versionmodified">New in version 1.0: </span>"""
-        u"""THIS IS THE <em>FIRST</em> PARAGRAPH OF VERSIONADDED.</p>\n""")
+        """<p><span class="versionmodified added">New in version 1.0: </span>"""
+        """THIS IS THE <em>FIRST</em> PARAGRAPH OF VERSIONADDED.</p>\n""")
     matched_content = get_content(result, "versionadded")
     assert expect2 == matched_content
 
     expect3 = (
-        u"""<p><span class="versionmodified">Changed in version 1.0: </span>"""
-        u"""THIS IS THE <em>FIRST</em> PARAGRAPH OF VERSIONCHANGED.</p>\n""")
+        """<p><span class="versionmodified changed">Changed in version 1.0: </span>"""
+        """THIS IS THE <em>FIRST</em> PARAGRAPH OF VERSIONCHANGED.</p>\n""")
     matched_content = get_content(result, "versionchanged")
     assert expect3 == matched_content
 
@@ -658,7 +734,7 @@ def test_html_docfields(app):
     app.build()
     # --- docfields
     # expect no error by build
-    (app.outdir / 'docfields.html').text(encoding='utf-8')
+    (app.outdir / 'docfields.html').text()
 
 
 @sphinx_intl
@@ -667,7 +743,7 @@ def test_html_docfields(app):
 def test_html_template(app):
     app.build()
     # --- gettext template
-    result = (app.outdir / 'index.html').text(encoding='utf-8')
+    result = (app.outdir / 'contents.html').text()
     assert "WELCOME" in result
     assert "SPHINX 2013.120" in result
 
@@ -679,12 +755,14 @@ def test_html_rebuild_mo(app):
     app.build()
     # --- rebuild by .mo mtime
     app.builder.build_update()
-    updated = app.env.update(app.config, app.srcdir, app.doctreedir, app)
+    app.env.find_files(app.config, app.builder)
+    _, updated, _ = app.env.get_outdated_files(config_changed=False)
     assert len(updated) == 0
 
     mtime = (app.srcdir / 'xx' / 'LC_MESSAGES' / 'bom.mo').stat().st_mtime
     (app.srcdir / 'xx' / 'LC_MESSAGES' / 'bom.mo').utime((mtime + 5, mtime + 5))
-    updated = app.env.update(app.config, app.srcdir, app.doctreedir, app)
+    app.env.find_files(app.config, app.builder)
+    _, updated, _ = app.env.get_outdated_files(config_changed=False)
     assert len(updated) == 1
 
 
@@ -701,9 +779,13 @@ def test_xml_footnotes(app, warning):
     assert_elem(
         para0[0],
         ['I18N WITH FOOTNOTE', 'INCLUDE THIS CONTENTS',
-         '2', '[ref]', '1', '100', '.'],
+         '2', '[ref]', '1', '100', '*', '. SECOND FOOTNOTE_REF', '100', '.'],
         ['i18n-with-footnote', 'ref'])
 
+    # check node_id for footnote_references which refer same footnote (refs: #3002)
+    assert para0[0][4].text == para0[0][6].text == '100'
+    assert para0[0][4].attrib['ids'] != para0[0][6].attrib['ids']
+
     footnote0 = secs[0].findall('footnote')
     assert_elem(
         footnote0[0],
@@ -720,6 +802,11 @@ def test_xml_footnotes(app, warning):
         ['2', 'THIS IS A AUTO NUMBERED NAMED FOOTNOTE.'],
         None,
         ['named'])
+    assert_elem(
+        footnote0[3],
+        ['*', 'THIS IS A AUTO SYMBOL FOOTNOTE.'],
+        None,
+        None)
 
     citation0 = secs[0].findall('citation')
     assert_elem(
@@ -729,7 +816,7 @@ def test_xml_footnotes(app, warning):
         ['ref'])
 
     warnings = getwarning(warning)
-    warning_expr = u'.*/footnote.xml:\\d*: SEVERE: Duplicate ID: ".*".\n'
+    warning_expr = '.*/footnote.xml:\\d*: SEVERE: Duplicate ID: ".*".\n'
     assert_not_re_search(warning_expr, warnings)
 
 
@@ -744,14 +831,13 @@ def test_xml_footnote_backlinks(app):
 
     para0 = secs[0].findall('paragraph')
     refs0 = para0[0].findall('footnote_reference')
-    refid2id = dict([
-        (r.attrib.get('refid'), r.attrib.get('ids')) for r in refs0])
+    refid2id = {r.attrib.get('refid'): r.attrib.get('ids') for r in refs0}
 
     footnote0 = secs[0].findall('footnote')
     for footnote in footnote0:
         ids = footnote.attrib.get('ids')
-        backrefs = footnote.attrib.get('backrefs')
-        assert refid2id[ids] == backrefs
+        backrefs = footnote.attrib.get('backrefs').split()
+        assert refid2id[ids] in backrefs
 
 
 @sphinx_intl
@@ -843,7 +929,7 @@ def test_xml_role_xref(app):
         para1,
         ['LINK TO', "I18N ROCK'N ROLE XREF", ',', 'CONTENTS', ',',
          'SOME NEW TERM', '.'],
-        ['i18n-role-xref', 'contents',
+        ['i18n-role-xref', 'index',
          'glossary_terms#term-some-term'])
 
     para2 = sec2.findall('paragraph')
@@ -860,7 +946,7 @@ def test_xml_role_xref(app):
     assert_elem(
         para2[2],
         ['LINK TO', 'I18N WITH GLOSSARY TERMS', 'AND', 'CONTENTS', '.'],
-        ['glossary_terms', 'contents'])
+        ['glossary_terms', 'index'])
     assert_elem(
         para2[3],
         ['LINK TO', '--module', 'AND', '-m', '.'],
@@ -954,7 +1040,7 @@ def test_xml_label_targets(app):
 def test_additional_targets_should_not_be_translated(app):
     app.build()
     # [literalblock.txt]
-    result = (app.outdir / 'literalblock.html').text(encoding='utf-8')
+    result = (app.outdir / 'literalblock.html').text()
 
     # title should be translated
     expected_expr = 'CODE-BLOCKS'
@@ -973,6 +1059,14 @@ def test_additional_targets_should_not_be_translated(app):
                      """<span class="cpf">&lt;stdio.h&gt;</span>""")
     assert_count(expected_expr, result, 1)
 
+    # literal block in list item should not be translated
+    expected_expr = ("""<span class="n">literal</span>"""
+                     """<span class="o">-</span>"""
+                     """<span class="n">block</span>\n"""
+                     """<span class="k">in</span> """
+                     """<span class="n">list</span>""")
+    assert_count(expected_expr, result, 1)
+
     # doctest block should not be translated but be highlighted
     expected_expr = (
         """<span class="gp">&gt;&gt;&gt; </span>"""
@@ -982,7 +1076,7 @@ def test_additional_targets_should_not_be_translated(app):
 
     # [raw.txt]
 
-    result = (app.outdir / 'raw.html').text(encoding='utf-8')
+    result = (app.outdir / 'raw.html').text()
 
     # raw block should not be translated
     expected_expr = """<iframe src="http://sphinx-doc.org"></iframe></div>"""
@@ -990,7 +1084,7 @@ def test_additional_targets_should_not_be_translated(app):
 
     # [figure.txt]
 
-    result = (app.outdir / 'figure.html').text(encoding='utf-8')
+    result = (app.outdir / 'figure.html').text()
 
     # alt and src for image block should not be translated
     expected_expr = """<img alt="i18n" src="_images/i18n.png" />"""
@@ -1020,7 +1114,7 @@ def test_additional_targets_should_not_be_translated(app):
 def test_additional_targets_should_be_translated(app):
     app.build()
     # [literalblock.txt]
-    result = (app.outdir / 'literalblock.html').text(encoding='utf-8')
+    result = (app.outdir / 'literalblock.html').text()
 
     # title should be translated
     expected_expr = 'CODE-BLOCKS'
@@ -1039,6 +1133,14 @@ def test_additional_targets_should_be_translated(app):
                      """<span class="cpf">&lt;STDIO.H&gt;</span>""")
     assert_count(expected_expr, result, 1)
 
+    # literal block in list item should be translated
+    expected_expr = ("""<span class="no">LITERAL</span>"""
+                     """<span class="o">-</span>"""
+                     """<span class="no">BLOCK</span>\n"""
+                     """<span class="no">IN</span> """
+                     """<span class="no">LIST</span>""")
+    assert_count(expected_expr, result, 1)
+
     # doctest block should not be translated but be highlighted
     expected_expr = (
         """<span class="gp">&gt;&gt;&gt; </span>"""
@@ -1048,7 +1150,7 @@ def test_additional_targets_should_be_translated(app):
 
     # [raw.txt]
 
-    result = (app.outdir / 'raw.html').text(encoding='utf-8')
+    result = (app.outdir / 'raw.html').text()
 
     # raw block should be translated
     expected_expr = """<iframe src="HTTP://SPHINX-DOC.ORG"></iframe></div>"""
@@ -1056,7 +1158,7 @@ def test_additional_targets_should_be_translated(app):
 
     # [figure.txt]
 
-    result = (app.outdir / 'figure.html').text(encoding='utf-8')
+    result = (app.outdir / 'figure.html').text()
 
     # alt and src for image block should be translated
     expected_expr = """<img alt="I18N -&gt; IMG" src="_images/img.png" />"""
@@ -1074,20 +1176,21 @@ def test_text_references(app, warning):
     app.builder.build_specific([app.srcdir / 'refs.txt'])
 
     warnings = warning.getvalue().replace(os.sep, '/')
-    warning_expr = u'refs.txt:\\d+: ERROR: Unknown target name:'
+    warning_expr = 'refs.txt:\\d+: ERROR: Unknown target name:'
     assert_count(warning_expr, warnings, 0)
 
 
 @pytest.mark.sphinx(
-    'dummy', testroot='image-glob',
-    srcdir='test_intl_image_glob',
+    'dummy', testroot='images',
+    srcdir='test_intl_images',
     confoverrides={'language': 'xx'}
 )
+@pytest.mark.xfail(os.name != 'posix', reason="Not working on windows")
 def test_image_glob_intl(app):
     app.build()
-    # index.rst
-    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
 
+    # index.rst
+    doctree = app.env.get_doctree('index')
     assert_node(doctree[0][1], nodes.image, uri='rimg.xx.png',
                 candidates={'*': 'rimg.xx.png'})
 
@@ -1107,8 +1210,7 @@ def test_image_glob_intl(app):
                             'image/png': 'img.png'})
 
     # subdir/index.rst
-    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
-
+    doctree = app.env.get_doctree('subdir/index')
     assert_node(doctree[0][1], nodes.image, uri='subdir/rimg.xx.png',
                 candidates={'*': 'subdir/rimg.xx.png'})
 
@@ -1123,18 +1225,19 @@ def test_image_glob_intl(app):
 
 
 @pytest.mark.sphinx(
-    'dummy', testroot='image-glob',
-    srcdir='test_intl_image_glob',
+    'dummy', testroot='images',
+    srcdir='test_intl_images',
     confoverrides={
         'language': 'xx',
-        'figure_language_filename': u'{root}{ext}.{language}',
+        'figure_language_filename': '{root}{ext}.{language}',
     }
 )
+@pytest.mark.xfail(os.name != 'posix', reason="Not working on windows")
 def test_image_glob_intl_using_figure_language_filename(app):
     app.build()
-    # index.rst
-    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
 
+    # index.rst
+    doctree = app.env.get_doctree('index')
     assert_node(doctree[0][1], nodes.image, uri='rimg.png.xx',
                 candidates={'*': 'rimg.png.xx'})
 
@@ -1154,8 +1257,7 @@ def test_image_glob_intl_using_figure_language_filename(app):
                             'image/png': 'img.png'})
 
     # subdir/index.rst
-    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
-
+    doctree = app.env.get_doctree('subdir/index')
     assert_node(doctree[0][1], nodes.image, uri='subdir/rimg.png',
                 candidates={'*': 'subdir/rimg.png'})
 
@@ -1170,4 +1272,4 @@ def test_image_glob_intl_using_figure_language_filename(app):
 
 
 def getwarning(warnings):
-    return warnings.getvalue().replace(os.sep, '/')
+    return strip_escseq(warnings.getvalue().replace(os.sep, '/'))
