@@ -1,5 +1,5 @@
 from numbers import Integral
-from typing import List, TypeVar, Union, Callable, Tuple, Optional
+from typing import Any, List, TypeVar, Union, Callable, Tuple, Optional
 
 
 def f0(x: int, y: Integral) -> None:
@@ -72,6 +72,22 @@ def f13() -> Optional[str]:
     pass
 
 
+def f14() -> Any:
+    pass
+
+
+def f15(x: "Unknown", y: "int") -> Any:
+    pass
+
+
+def f16(arg1, arg2, *, arg3=None, arg4=None):
+    pass
+
+
+def f17(*, arg3, arg4):
+    pass
+
+
 class Node:
     def __init__(self, parent: Optional['Node']) -> None:
         pass
